package com.project.zcustom.service.build;

import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.project.zcustom.controller.core.PageParamVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.project.zcustom.domain.build.PlatBuildParams;

import java.util.List;
import java.util.Map;

/**
 * 楼宇数据Service接口
 *
 * @author change
 * @date 2024-11-01
 */
public interface IPlatBuildParamsService extends IService<PlatBuildParams> {

    /**
     * 分页查询楼宇数据列表
     *
     * @param platBuildParams 楼宇数据
     * @return 楼宇数据集合
     */
    public IPage<PlatBuildParams> pagingPlatBuildParamsList(PlatBuildParams platBuildParams);

    /**
     * 分页查询楼宇数据列表
     *
     * @param lqw
     * @return 楼宇数据集合
     */
    public IPage<PlatBuildParams> pagingPlatBuildParamsList(PageParamVo pageParamVo, LambdaQueryWrapper<PlatBuildParams> lqw);

    /**
     * 查询楼宇数据列表
     *
     * @param platBuildParams 楼宇数据
     * @return 楼宇数据集合
     */
    public List<PlatBuildParams> selectPlatBuildParamsList(PlatBuildParams platBuildParams);

    /**
     * 查询楼宇数据列表
     *
     * @param platBuildParams 楼宇数据
     * @return 楼宇数据集合
     */
    public List<PlatBuildParams> selectPlatBuildParamsList(PlatBuildParams platBuildParams,Long limitNum);

    /**
     * 根据指定字段查询信息
     *
     * @param field 字段名称
     * @param filedValue 字段值
     * @return 楼宇数据
     */
    public PlatBuildParams getPlatBuildParamsByField(String field, String filedValue);

    /**
     * 根据指定字段查询信息列表
     *
     * @param field 字段名称
     * @param filedValue 字段值
     * @return 楼宇数据
     */
    public List<PlatBuildParams> selectPlatBuildParamsListByField(String field, String filedValue);

    /**
     * 根据指定字段查询信息列表
     *
     * @param field 字段名称
     * @param filedValue 字段值
     * @return 楼宇数据
     */
    public List<PlatBuildParams> selectPlatBuildParamsListByField(String field, String filedValue,Long limitNum);

    /**
     *判断指定字段值是否存在
     *
     * @param field 字段名称
     * @param filedValue 字段值
     */
    public void checkFieldExist(String field, String filedValue,String tips,Long id);

    /**
     *  真删接口
     */
    public void reallyDeletePlatBuildParams(LambdaQueryWrapper<PlatBuildParams> lqw);

    /**
     * 设置添加趋势数据
     */
    public void setBuildParamsData(PlatBuildParams platBuildParams, List<String> keyList, Map<String, Object> mapData, Integer type, String appOrg);

    /**
     * 添加楼宇数据
     */
    public void addPlatBuildParams(PlatBuildParams platBuildParams,String appOrg);

    /**
     * 月用电量信息
     * @return
     */
    public Map<String, Object> powerForMonth();

    /**
     * 年度累计综合能耗
     * @return
     */
    public Map<String, Object> energyForYear();


    /**
     * 楼宇月用水电热信息
     */
    public Map<String, Object> useHeatGas(String appOrg);

    /**
     * 数据中心机房信息接口
     */
    public Map<String, Object> dataCenterRoom(String appOrg);

    /**
     * 年度累计电耗
     */
    public Map<String, Object> powerForYear();

    /**
     * 供暖能耗信息
     */
    public Map<String, Object> heatingEnergy(String appOrg);

    /**
     * 燃气用量信息
     */
    public Map<String, Object> gasInfo(String appOrg);

    /**
     * 资源信息
     */
    public List<Map<String, Object>> resourceInfo(String appOrg);

    /**
     * 可再生电力信息
     */
    public Map<String, Object> greenPowerInfo(String appOrg);

    /**
     * 年度累计水耗
     */
    public Map<String, Object> waterForYear();

    /**
     * 办公碳排放情况
     */
    public Map<String, Object> officeCarbonForYear();

    /**
     * 年度累计非供暖能耗
     */
    public Map<String, Object> nonHeatingEnergyForYear();

}
